% This code accompanies the paper:
%
% "Elastic Net Constraints for Shape Matching"
% E.Rodola, A.Torsello, T.Harada, Y.Kuniyoshi, and D.Cremers
% 2013 IEEE International Conference on Computer Vision (ICCV 2013)
%
% Please cite the paper above if you use this code.
%
% Emanuele Rodola
% The University of Tokyo, 2012
% TU Munich, 2013
% <rodola@in.tum.de>

addpath('./tools/');

% Load the shapes and compute descriptors for each point

shape1 = load('./data/shape1.mat');
shape1 = shape1.shape;

shape2 = load('./data/shape2.mat');
shape2 = shape2.shape;

shape1.WKS = calc_WKS(shape1, 50, 6);
shape2.WKS = calc_WKS(shape2, 50, 6);

% Create candidate matches by looking for closest points in descriptor
% space (this will produce several mismatches).

samples1 = fps_euclidean(shape1.VERT, 200, 1);

tree = ann('init', shape2.WKS');
samples2 = ann('search', tree, shape1.WKS(samples1,:)', 1)';
ann('deinit', tree);
clear tree

candidate_matches = [samples1 samples2];

% Similarity matrix
% (Here you can plug-in your own similarity matrix)

S = sim_matrix(shape1, shape2, candidate_matches);

% Solve via elastic net-projected gradient descent

figure
alphas = 0.1:0.1:0.9;

for a=1:length(alphas)
    [x, iters, err] = enet_grad_desc(S, alphas(a), [], 1e-5, 1e4, 1e-4);
    
    x(x<0.05*max(x)) = 0;
    % WARNING: make sure prune_matches() receives variables of type DOUBLE
    matches = prune_matches(x, double(candidate_matches-1)');
    
    fprintf('Alpha: %.2f, Iterations: %d, Error: %.2e, Matches: %d\n', alphas(a), iters, err, size(matches,1));

    subplot(3,3,a)
    plot_matches(shape1, shape2, candidate_matches(matches,:), alphas(a))
end
